// Copyright by Uri Siman, Delphi 2 main module, QR HTML export filter
unit ProHtml;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  QuickRep, ComCtrls;

const Siz = 50; //max count fields in row
      SizY = 40; //max (band) height of page in Pixels/ab1

type
   TOnPrintEvent = procedure(Sender : TObject; var Value : String) of object;
   TAfterPrintEvent = procedure(BandPrinted : Boolean) of object;
   TRepBeforePrintEvent = procedure(var ReportPrint : Boolean) of object;
   TRepAfterPrintEvent = procedure(Sender: TObject) of object;

SendPageHTMLOn = Class
 OldCall : Array[0..Siz*5] of TOnPrintEvent;
 PrnBand : Array[0..Siz*5] of TAfterPrintEvent;
 end;

type
  THTMLParams = class (TPersistent)
   private
    FIgnorePageHeader : Boolean;
    FIgnorePageFooter : Boolean;
    FFileName : String;
    FBorderSize : Word;
    FScaleFontSize : Word;
    FFontCoef : Word;
   published
    property IgnorePageHeader : Boolean read FIgnorePageHeader write FIgnorePageHeader;
    property IgnorePageFooter : Boolean read FIgnorePageFooter write FIgnorePageFooter;
    property FileName : String read FFileName write FFileName;
    property BorderSize : Word read FBorderSize write FBorderSize default 1;
    property ScaleFontSize : Word read FScaleFontSize write FScaleFontSize default 100;
    property FontCoef : Word read FFontCoef write FFontCoef default 4;
  end;

type
  TStencil = class (TPersistent)
   private
    FHorizontalSensitiv : Word;
    FVerticalSensitiv : Word;
   published
    property VerticalSensitiv : Word read FVerticalSensitiv write FVerticalSensitiv default 14;
    property HorizontalSensitiv : Word read FHorizontalSensitiv write FHorizontalSensitiv default 3;
  end;

type
  TTabBorders = class (TPersistent)
   private
    FBorderInside : Boolean;
    FBorderOutside : Boolean;
    FDetailToTable : Boolean;
    FTitleToTable : Boolean;
    FPageHeaderToTable : Boolean;
    FPageFooterToTable : Boolean;
    FGroupHeaderToTable : Boolean;
    FGroupFooterToTable : Boolean;
    FSubDetailToTable : Boolean;
    FColumnHeaderToTable : Boolean;
    FOverlayToTable : Boolean;
    FSummaryToTable : Boolean;
   published
    property BorderInside : Boolean read FBorderInside write FBorderInside;
    property BorderOutside : Boolean read FBorderOutside write FBorderOutside;
    property DetailToTable : Boolean read FDetailToTable write FDetailToTable;
    property TitleToTable : Boolean read FTitleToTable write FTitleToTable;
    property PageHeaderToTable : Boolean read FPageHeaderToTable write FPageHeaderToTable;
    property PageFooterToTable : Boolean read FPageFooterToTable write FPageFooterToTable;
    property GroupHeaderToTable : Boolean read FGroupHeaderToTable write FGroupHeaderToTable;
    property GroupFooterToTable : Boolean read FGroupFooterToTable write FGroupFooterToTable;
    property SubDetailToTable : Boolean read FSubDetailToTable write FSubDetailToTable;
    property ColumnHeaderToTable : Boolean read FColumnHeaderToTable write FColumnHeaderToTable;
    property OverlayToTable : Boolean read FOverlayToTable write FOverlayToTable;
    property SummaryToTable : Boolean read FSummaryToTable write FSummaryToTable;
  end;

type
  TCharSetName = (Auto,
                  ISO_8859_1,
                  ISO_8859_2,
                  ISO_8859_5,
                  ISO_8859_7,
                  ISO_8859_9,
                  Windows_1250,
                  Windows_1251,
                  Windows_1253,
                  KOI8_R,
                  Shift_JIS,
                  EUC_JP,
                  Big5,
                  EUC_TW,
                  GB2312,
                  UTF_8,
                  UTF_7);


  TDRepHTML = class(TComponent)
  private
    { Private declarations }
    FExecute : Boolean;
    FHTMLCharSet : TCharSetName;
    FTitle : String;
    FHTMLParams : THTMLParams;
    FStencil : TStencil;
    FTabBorders : TTabBorders;
    FShowProgress : Boolean;
  protected
    { Protected declarations }
    procedure SetExecute(Value : boolean);
  public
    { Public declarations }
    PageHTMLOn : SendPageHTMLOn;
    procedure QRLabel1Print(sender: TObject; var Value: string);
    procedure QRBand1AfterPrint(BandPrinted: Boolean);
    procedure APrint(sender: TObject);
    procedure BPrint(var PrintReport : Boolean);
    procedure Loaded; override;
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property HTMLCharSet : TCharSetName read FHTMLCharSet write FHTMLCharSet;
    property Title : String read FTitle write FTitle;
    property Execute : Boolean read FExecute write SetExecute;
    property HTMLParams : THTMLParams read FHTMLParams write FHTMLParams;
    property Stencil : TStencil read FStencil write FStencil;
    property TabBorders : TTabBorders read FTabBorders write FTabBorders;
    property ShowProgress : Boolean read FShowProgress write FShowProgress;
  end;

var ExecuteToHTML : Boolean;
    Bevent        : Boolean;
    tqr : TQuickReport;
fm, gfm : TForm;
    mpb : TProgressBar;
    Ab1 : Word; {14 : Vertical Sensitivity in Pixels}
    Ab2 : Word; {10 : Horizontal Sensitivity in Pixels, 5-optimal}
 OldRef : Word;
// RefCnt : Word;

procedure Register;

Function PagePack(Y, L : Integer) : Word;

implementation
uses Printers, ProSHtml;

Var
   BefPrn : TRepBeforePrintEvent;
   AftPrn : TRepAfterPrintEvent;

constructor TDRepHTML.Create(AOwner : TComponent);
begin
 inherited Create(AOwner);
 FStencil:=TStencil.Create;
 FHTMLParams:=THTMLParams.Create;
 FTabBorders:=TTabBorders.Create;

 FHTMLCharSet:=ISO_8859_1;

 FTitle:='Report';
 FShowProgress:=True;

 With FStencil Do
 Begin
  FVerticalSensitiv:= 14;
  FHorizontalSensitiv:= 10;
 end;

 With FHTMLParams Do
 Begin
  FIgnorePageHeader:= True;
  FIgnorePageFooter:= True;
  FFileName:= 'Report.htm';
  FBorderSize:= 1;
  FScaleFontSize:= 100;
  FFontCoef:= 4;
 end;

 With FTabBorders Do
 Begin
  FBorderInside:= False;
  FBorderOutSide:= False;
  FDetailToTable:= True;
  FTitleToTable:= False;
  FPageHeaderToTable:= False;
  FPageFooterToTable:= False;
  FGroupHeaderToTable:= False;
  FGroupFooterToTable:= False;
  FSubDetailToTable:= False;
  FColumnHeaderToTable:= False;
  FOverlayToTable:= False;
  FSummaryToTable:= False;
 end;

// RefCnt :=0; //nothing row of tables before
 OldRef :=0; //nothing band before
 PrnPage :=0; //printed pages

end;

destructor TDRepHTML.Destroy;
begin
 ExecuteToHTML:=FALSE;
 QRPrinter.CleanUp;

 FStencil.Free;
 FHTMLParams.Free;

 inherited Destroy;
end;

procedure TDRepHTML.SetExecute(Value : boolean);
var frm : TComponent;
      i : Integer;
begin
If Value<>FExecute then
begin
    FExecute:=Value;

If FExecute then
begin

 Ab1:=FStencil.FVerticalSensitiv;
 Ab2:=FStencil.FHorizontalSensitiv;

 EXBP:=False;
 EXAP:=False;
 BEvent:=TRUE;
 frm:=Owner;
 while (not (frm is TForm)) do frm:=frm.Owner;
 fm:=(frm as TForm);
 with fm do
 begin
  i:=0;
  while (i<ComponentCount) do
    if (Components[i] is TQuickReport) then
     begin
      tqr:=(Components[i] as TQuickReport);
      If Assigned((Components[i] as TQuickReport).BeforePrint) then
         EXBP:=True; {saving old call BeforePrint}
      If Assigned((Components[i] as TQuickReport).AfterPrint) then
         EXAP:=True; {saving old call AfterPrint}

      AftPrn:=tqr.AfterPrint;
      BefPrn:=tqr.BeforePrint;
      tqr.BeforePrint:=BPrint;
      tqr.AfterPrint:=APrint;

      tqr.Prepare;
      FExecute:=False;

      Break;
     end
    else Inc(i);
 end;
end;
end;
end;

Function PagePack(Y, L : Integer) : Word;
Var I, J : Integer;
Begin
 For I:=0 to PageHTML.Empty[Y]-1 Do
 If PageHTML.Left[I, Y] >= L then
 Begin
   For J:=PageHTML.Empty[Y] Downto I+1 Do
   Begin
    PageHTML.WLine[J, Y]:= PageHTML.WLine[J-1, Y];
    PageHTML.Top[J, Y]:= PageHTML.Top[J-1, Y];
    PageHTML.Left[J, Y]:= PageHTML.Left[J-1, Y];
    PageHTML.Width[J, Y]:= PageHTML.Width[J-1, Y];
    PageHTML.Align[J, Y]:= PageHTML.Align[J-1, Y];
    PageHTML.FName[J, Y]:= PageHTML.FName[J-1, Y];
    PageHTML.FSize[J, Y]:= PageHTML.FSize[J-1, Y];
    PageHTML.FStyle[J, Y]:= PageHTML.FStyle[J-1, Y];
   end;

   If PageHTML.Empty[Y]<Siz then
      PageHTML.Empty[Y]:=PageHTML.Empty[Y]+1;
   Result:=I;
   Exit;
 end;

 If PageHTML.Empty[Y]<Siz then
    PageHTML.Empty[Y]:=PageHTML.Empty[Y]+1;

 Result:=PageHTML.Empty[Y] - 1;
end;

procedure TDRepHTML.QRLabel1Print(sender: TObject; var Value: string);
var str : string;
   X, Y : Integer;
    Pix : Integer;
     Tg : LongInt;
begin

If Length(Value)<=0 then
If ((TQRBand((Sender As Tcontrol).Parent).BandType=rbTitle)
  And (FTabBorders.FTitleToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbDetail)
  And (FTabBorders.FDetailToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbPageHeader)
  And (FTabBorders.FPageHeaderToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbPageFooter)
  And (FTabBorders.FPageFooterToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbGroupHeader)
  And (FTabBorders.FGroupHeaderToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbGroupFooter)
  And (FTabBorders.FGroupFooterToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbSubDetail)
  And (FTabBorders.FSubDetailToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbColumnHeader)
  And (FTabBorders.FColumnHeaderToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbOverlay)
  And (FTabBorders.FOverlayToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbSummary)
  And (FTabBorders.FSummaryToTable=True)) then
  Value:=' ';

 Case PageHTML.Typ[(Sender As TComponent).Tag] of
 1: If Assigned(PageHTMLOn.OldCall[(Sender As TQRLabel).Tag]) then
    Begin
     {  ,  }
     (Sender As TQRLabel).OnPrint:=
                 PageHTMLOn.OldCall[(Sender As TQRLabel).Tag];
      Tg:=(Sender As TQRLabel).Tag; {  }
     (Sender As TQRLabel).Tag:= PageHTML.OldTag[Tg];
     {  }
     (Sender As TQRLabel).OnPrint(sender, Value);
     (Sender As TQRLabel).Tag:=Tg;
    end;
 2: If Assigned(PageHTMLOn.OldCall[(Sender As TQRDBText).Tag]) then
    Begin
     {  ,  }
     (Sender As TQRDBText).OnPrint:=
                 PageHTMLOn.OldCall[(Sender As TQRDBText).Tag];
      Tg:=(Sender As TQRDBText).Tag; {  }
     (Sender As TQRDBText).Tag:= PageHTML.OldTag[Tg];
     {  }
     (Sender As TQRDBText).OnPrint(sender, Value);
     (Sender As TQRDBText).Tag:=Tg;
    end;
 3: If Assigned(PageHTMLOn.OldCall[(Sender As TQRDBCalc).Tag]) then
    Begin
     {  ,  }
     (Sender As TQRDBCalc).OnPrint:=
                 PageHTMLOn.OldCall[(Sender As TQRDBCalc).Tag];
      Tg:=(Sender As TQRDBCalc).Tag; {  }
     (Sender As TQRDBCalc).Tag:= PageHTML.OldTag[Tg];
     {  }
     (Sender As TQRDBCalc).OnPrint(sender, Value);
     (Sender As TQRDBCalc).Tag:=Tg;
    end;
 4: If Assigned(PageHTMLOn.OldCall[(Sender As TQRSysData).Tag]) then
    Begin
     {  ,  }
     (Sender As TQRSysData).OnPrint:=
                 PageHTMLOn.OldCall[(Sender As TQRSysData).Tag];
      Tg:=(Sender As TQRSysData).Tag; {  }
     (Sender As TQRSysData).Tag:= PageHTML.OldTag[Tg];
     {  }
     (Sender As TQRSysData).OnPrint(sender, Value);
     (Sender As TQRSysData).Tag:=Tg;
    end;
  end;

  Ref:=PageHTML.Band[(Sender As TComponent).Tag];

  If (Sender Is TQRLabel) then
  Begin
   {   }
   Y:= (Sender As TQRLabel).Top div Ab1;
   X:= (Sender As TQRLabel).Left div Ab2;
   PageHTML.Name[(Sender As TComponent).Tag]:=
                 (Sender As TQRLabel).Name;

   X:=PagePack(Y, (Sender As TQRLabel).Left);

   PageHTML.WLine[X, Y]:= Value;
   PageHTML.Top[X, Y]:= (Sender As TQRLabel).Top;
   PageHTML.Left[X, Y]:= (Sender As TQRLabel).Left;
   PageHTML.Width[X, Y]:= (Sender As TQRLabel).Width;
   PageHTML.Align[X, Y]:= (Sender As TQRLabel).Alignment;
   PageHTML.FName[X, Y]:= (Sender As TQRLabel).Font.Name;
   PageHTML.FSize[X, Y]:= (Sender As TQRLabel).Font.Size;
   PageHTML.FStyle[X, Y]:= (Sender As TQRLabel).Font.Style;
   end;

  If (Sender Is TQRDBText) then
  Begin
   {   }
   Y:= (Sender As TQRDBText).Top div Ab1;
   X:= (Sender As TQRDBText).Left div Ab2;
   PageHTML.Name[(Sender As TComponent).Tag]:=
                 (Sender As TQRDBText).Name;

   X:=PagePack(Y, (Sender As TQRDBText).Left);

   PageHTML.WLine[X, Y]:= Value;
   PageHTML.Top[X, Y]:= (Sender As TQRDBText).Top;
   PageHTML.Left[X, Y]:= (Sender As TQRDBText).Left;
   PageHTML.Width[X, Y]:= (Sender As TQRDBText).Width;
   PageHTML.Align[X, Y]:= (Sender As TQRDBText).Alignment;
   PageHTML.FName[X, Y]:= (Sender As TQRDBText).Font.Name;
   PageHTML.FSize[X, Y]:= (Sender As TQRDBText).Font.Size;
   PageHTML.FStyle[X, Y]:= (Sender As TQRDBText).Font.Style;
  end;

  If (Sender Is TQRDBCalc) then
  Begin
   {   }
   Y:= (Sender As TQRDBCalc).Top div Ab1;
   X:= (Sender As TQRDBCalc).Left div Ab2;
   PageHTML.Name[(Sender As TComponent).Tag]:=
                 (Sender As TQRDBCalc).Name;

   X:=PagePack(Y, (Sender As TQRDBCalc).Left);

   PageHTML.WLine[X, Y]:= Value;
   PageHTML.Top[X, Y]:= (Sender As TQRDBCalc).Top;
   PageHTML.Left[X, Y]:= (Sender As TQRDBCalc).Left;
   PageHTML.Width[X, Y]:= (Sender As TQRDBCalc).Width;
   PageHTML.Align[X, Y]:= (Sender As TQRDBCalc).Alignment;
   PageHTML.FName[X, Y]:= (Sender As TQRDBCalc).Font.Name;
   PageHTML.FSize[X, Y]:= (Sender As TQRDBCalc).Font.Size;
   PageHTML.FStyle[X, Y]:= (Sender As TQRDBCalc).Font.Style;
   end;

  If (Sender Is TQRSysData) then
  Begin
   {   }
   Y:= (Sender As TQRSysData).Top div Ab1;
   X:= (Sender As TQRSysData).Left div Ab2;
   PageHTML.Name[(Sender As TComponent).Tag]:=
                 (Sender As TQRSysData).Name;

   X:=PagePack(Y, (Sender As TQRSysData).Left);

   PageHTML.WLine[X, Y]:= Value;
   PageHTML.Top[X, Y]:= (Sender As TQRSysData).Top;
   PageHTML.Left[X, Y]:= (Sender As TQRSysData).Left;
   PageHTML.Width[X, Y]:= (Sender As TQRSysData).Width;
   PageHTML.Align[X, Y]:= (Sender As TQRSysData).Alignment;
   PageHTML.FName[X, Y]:= (Sender As TQRSysData).Font.Name;
   PageHTML.FSize[X, Y]:= (Sender As TQRSysData).Font.Size;
   PageHTML.FStyle[X, Y]:= (Sender As TQRSysData).Font.Style;
   end;
  try
   RefW:=TQRBand((Sender As Tcontrol).Parent).Height;
  except
   RefW:=50000; //band not exist for this control
  end;
end;

procedure TDRepHTML.QRBand1AfterPrint(BandPrinted: Boolean);
begin
   SubFHorizontalSensitiv:= FStencil.FHorizontalSensitiv;
   SubFBorderSize:= FHTMLParams.FBorderSize;
   SubFScaleFontSize:= FHTMLParams.FScaleFontSize;
   SubFFontCoef:=FHTMLParams.FFontCoef;

   SubFTitleToTable:=FTabBorders.FTitleToTable;
   SubFDetailToTable:=FTabBorders.FDetailToTable;
   SubFPageHeaderToTable:=FTabBorders.FPageHeaderToTable;
   SubFPageFooterToTable:=FTabBorders.FPageFooterToTable;
   SubFGroupHeaderToTable:=FTabBorders.FGroupHeaderToTable;
   SubFGroupFooterToTable:=FTabBorders.FGroupFooterToTable;
   SubFSubDetailToTable:=FTabBorders.FSubDetailToTable;
   SubFColumnHeaderToTable:=FTabBorders.FColumnHeaderToTable;
   SubFOverlayToTable:=FTabBorders.FOverlayToTable;
   SubFSummaryToTable:=FTabBorders.FSummaryToTable;

   SubIgnorePageHeader:=FHTMLParams.FIgnorePageHeader;
   SubIgnorePageFooter:=FHTMLParams.FIgnorePageFooter;

   SubBorderInside:=FTabBorders.BorderInside;
   SubBorderOutside:=FTabBorders.BorderOutside;

   OnAftPrn; {calling transfer method}

   If (Ref=2) and FShowProgress then
   With Mpb Do
     If Position + Step <= Max then
     begin
        Position:=Position + Step;
        Show;
     end;
end;

procedure TDRepHTML.APrint(sender : TObject);
var   i : integer;
begin

 if FExecute=False then Exit;

 WriteLn(aFile, #13);
 WriteLn(aFile, '</HTML>');
 CloseFile(aFile);

 Gfm.Free;

 I:=0;
 with fm do
 while (i<ComponentCount) do
 if (Components[i] is TQuickReport) then
 begin {calling old afterprint}
    (Components[i] As TQuickReport).AfterPrint:=AftPrn;
    If EXAP then
       (Components[i] As TQuickReport).AfterPrint(sender);
    Break;
 end
 else Inc(i);

end;


procedure TDRepHTML.Loaded;
begin
 inherited Loaded;

// if FExecute=False then Exit;

end;

procedure TDRepHTML.BPrint(var PrintReport : Boolean);
Var I, J : Integer;
str,str1 : string;
begin

 if FExecute=False then Exit;

  I:=0;
  with fm do
  while (i<ComponentCount) do
  if (Components[i] is TQuickReport) then
  begin {  beforeprint}
    (Components[i] As TQuickReport).BeforePrint:=BefPrn;
    If EXBP then
       (Components[i] As TQuickReport).BeforePrint(PrintReport);

 If FShowProgress then
 begin
 Gfm:= TForm.Create(Application);
 With Gfm Do
 Begin
  Height:=50;
  Width:=300;
  BorderStyle:=bsSingle;
  Caption:='Progress...';
  Position:=poScreenCenter;
 end;

 Mpb:=TProgressBar.Create(nil);
 Mpb.Parent:=Gfm;
 With Mpb Do
 Begin
  Align:=alClient;
  Step:=1;
  Max:= (fm.Components[I] As TQuickReport).DataSource.DataSet.RecordCount;
  Position:=0;
  Visible:=True;
 end;
  Gfm.Show;
  Gfm.BringToFront;
  Gfm.SetFocus;
 end;

    Break;
  end
  else Inc(i);

  if Not BEvent then exit;
  if  FExecute then
  begin

   Ref:=0;
   L:=0;

  AssignFile(aFile, FHTMLParams.FFileName);

  try
   Rewrite(aFile);
  except
   ShowMessage('Sorry!.. Can`t create HTML !!!');
  end;

  If FHTMLCharSet=ISO_8859_1 then str:='ISO-8859-1' else
  If FHTMLCharSet=ISO_8859_2 then str:='ISO-8859-2' else
  If FHTMLCharSet=ISO_8859_5 then str:='ISO-8859-5' else
  If FHTMLCharSet=ISO_8859_7 then str:='ISO-8859-7' else
  If FHTMLCharSet=ISO_8859_9 then str:='ISO-8859-9' else
  If FHTMLCharSet=Windows_1250 then str:='Windows-1250' else
  If FHTMLCharSet=Windows_1251 then str:='Windows-1251' else
  If FHTMLCharSet=Windows_1253 then str:='Windows-1253' else
  If FHTMLCharSet=KOI8_R then str:='KOI8-R' else
  If FHTMLCharSet=Shift_JIS then str:='Shift_JIS' else
  If FHTMLCharSet=EUC_JP then str:='EUC-JP' else
  If FHTMLCharSet=Big5 then str:='Big5' else
  If FHTMLCharSet=EUC_TW then str:='EUC-TW' else
  If FHTMLCharSet=GB2312 then str:='GB2312' else
  If FHTMLCharSet=UTF_8 then str:='UTF-8' else
  If FHTMLCharSet=UTF_7 then str:='UTF-7'
  else  str:='Auto';


  If FTitle='' then str1:= FHTMLParams.FFileName
               else str1:= FTitle;

  WriteLn(aFile, '<HTML>');

  WriteLn(aFile, '<HEAD>');
  WriteLn(aFile, ' <TITLE> ' + str1 + ' </TITLE>');
  WriteLn(aFile, ' <META HTTP-EQUIV="Content-Type" CONTENT="TEXT/HTML; CHARSET='+str+'">');
  WriteLn(aFile, '</HEAD>');


   PageHTML:=SendPageHTML.Create;
   PageHTMLOn:=SendPageHTMLOn.Create;

   J:=0;

   For I:=0 to fm.ComponentCount - 1 Do
   Begin
    If (fm.Components[I] Is TQRBand) then
    Begin   {   }
     PageHTMLOn.PrnBand[J]:=(fm.Components[I] As TQRBand).AfterPrint;
     (fm.Components[I] As TQRBand).AfterPrint:=QRBand1AfterPrint;
    end;

    If (fm.Components[I] Is TQRLabel) then
    Begin
     PageHTML.OldTag[J]:=fm.Components[I].Tag;
     PageHTML.Typ[J]:=1;
     Case ((fm.Components[I] As TQRLabel).Parent As TQRBand).BandType of
      rbTitle        : PageHTML.Band[J]:=1;
      rbDetail       : PageHTML.Band[J]:=2;
      rbPageHeader   : PageHTML.Band[J]:=3;
      rbPageFooter   : PageHTML.Band[J]:=4;
      rbGroupHeader  : PageHTML.Band[J]:=5;
      rbGroupFooter  : PageHTML.Band[J]:=6;
      rbSubDetail    : PageHTML.Band[J]:=7;
      rbColumnHeader : PageHTML.Band[J]:=8;
      rbOverlay      : PageHTML.Band[J]:=9;
      rbSummary      : PageHTML.Band[J]:=10;
     else
      PageHTML.Band[J]:=10;
     end;
     fm.Components[I].Tag:=J; {Labels}
     PageHTMLOn.OldCall[J]:=TQRLabel(fm.Components[I]).OnPrint;
     TQRLabel(fm.Components[I]).OnPrint:=QRLabel1Print;
     Inc(J);
    end;
    If (fm.Components[I] Is TQRDBText) then
    Begin
     PageHTML.OldTag[J]:=fm.Components[I].Tag;
     PageHTML.Typ[J]:=2;
     Case ((fm.Components[I] As TQRDBText).Parent As TQRBand).BandType of
      rbTitle        : PageHTML.Band[J]:=1;
      rbDetail       : PageHTML.Band[J]:=2;
      rbPageHeader   : PageHTML.Band[J]:=3;
      rbPageFooter   : PageHTML.Band[J]:=4;
      rbGroupHeader  : PageHTML.Band[J]:=5;
      rbGroupFooter  : PageHTML.Band[J]:=6;
      rbSubDetail    : PageHTML.Band[J]:=7;
      rbColumnHeader : PageHTML.Band[J]:=8;
      rbOverlay      : PageHTML.Band[J]:=9;
      rbSummary      : PageHTML.Band[J]:=10;
     else
      PageHTML.Band[J]:=10;
     end;
     fm.Components[I].Tag:=J; {DBTexts}
     PageHTMLOn.OldCall[J]:=TQRDBText(fm.Components[I]).OnPrint;
     TQRDBText(fm.Components[I]).OnPrint:=QRLabel1Print;
     Inc(J);
    end;
    If (fm.Components[I] Is TQRDBCalc) then
    Begin
     PageHTML.OldTag[J]:=fm.Components[I].Tag;
     PageHTML.Typ[J]:=3;
     Case ((fm.Components[I] As TQRDBCalc).Parent As TQRBand).BandType of
      rbTitle        : PageHTML.Band[J]:=1;
      rbDetail       : PageHTML.Band[J]:=2;
      rbPageHeader   : PageHTML.Band[J]:=3;
      rbPageFooter   : PageHTML.Band[J]:=4;
      rbGroupHeader  : PageHTML.Band[J]:=5;
      rbGroupFooter  : PageHTML.Band[J]:=6;
      rbSubDetail    : PageHTML.Band[J]:=7;
      rbColumnHeader : PageHTML.Band[J]:=8;
      rbOverlay      : PageHTML.Band[J]:=9;
      rbSummary      : PageHTML.Band[J]:=10;
     else
      PageHTML.Band[J]:=10;
     end;
     fm.Components[I].Tag:=J; {DBCalc}
     PageHTMLOn.OldCall[J]:=TQRDBCalc(fm.Components[I]).OnPrint;
     TQRDBCalc(fm.Components[I]).OnPrint:=QRLabel1Print;
     Inc(J);
    end;
    If (fm.Components[I] Is TQRSysData) then
    Begin
     PageHTML.OldTag[J]:=fm.Components[I].Tag;
     PageHTML.Typ[J]:=4;
     Case ((fm.Components[I] As TQRSysData).Parent As TQRBand).BandType of
      rbTitle        : PageHTML.Band[J]:=1;
      rbDetail       : PageHTML.Band[J]:=2;
      rbPageHeader   : PageHTML.Band[J]:=3;
      rbPageFooter   : PageHTML.Band[J]:=4;
      rbGroupHeader  : PageHTML.Band[J]:=5;
      rbGroupFooter  : PageHTML.Band[J]:=6;
      rbSubDetail    : PageHTML.Band[J]:=7;
      rbColumnHeader : PageHTML.Band[J]:=8;
      rbOverlay      : PageHTML.Band[J]:=9;
      rbSummary      : PageHTML.Band[J]:=10;
     else
      PageHTML.Band[J]:=10;
     end;
     fm.Components[I].Tag:=J; {SysData}
     PageHTMLOn.OldCall[J]:=TQRSysData(fm.Components[I]).OnPrint;
     TQRSysData(fm.Components[I]).OnPrint:=QRLabel1Print;
     Inc(J);
    end;
   end;
  end;
 BEvent:=FALSE;

end;

procedure Register;
begin
  RegisterComponents('QReport', [TDRepHTML]);
end;

end.
